// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.logging;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.logging.ILogger;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.http.Request;
import org.opengroup.osdu.notification.di.RequestInfoExt;

import javax.servlet.FilterChain;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyMap;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ResponseLogFilterTest {

    @InjectMocks
    ResponseLogFilter responseLogFilter;

    @Mock
    private RequestInfoExt requestInfoExt;

    @Mock
    private ILogger logger;

    @Mock
    private FilterChain filterChain;

    @Mock
    private ServletContext servletContext;

    @Mock
    private HttpServletRequest servletRequest;

    @Mock
    private HttpServletResponse servletResponse;

    @Spy
    private DpsHeaders dpsHeaders;

    @Before
    public void setup() throws ServletException, IOException {
        MockitoAnnotations.initMocks(ResponseLogFilterTest.this);
        when(servletRequest.getServletContext()).thenReturn(servletContext);
    }

    @Test
    public void doFilterTest() throws ServletException, IOException {
        when(requestInfoExt.getHeaders()).thenReturn(dpsHeaders);
        when(servletRequest.getMethod()).thenReturn("OPTIONS");
        responseLogFilter.doFilter(servletRequest, servletResponse, filterChain);
        verify(logger, times(1)).request(eq("notification.request"), any(Request.class), anyMap());
    }
}
